package net.sergeych.kiloparsec

import io.ktor.server.engine.*
import io.ktor.server.netty.*
import kotlinx.coroutines.test.runTest
import net.sergeych.crypto2.initCrypto
import net.sergeych.kiloparsec.adapter.acceptTcpDevice
import net.sergeych.kiloparsec.adapter.connectTcpDevice
import net.sergeych.kiloparsec.adapter.setupWebsocketServer
import net.sergeych.kiloparsec.adapter.websocketClient
import net.sergeych.mp_logger.Log
import java.net.InetAddress
import kotlin.test.Test
import kotlin.test.assertEquals
import kotlin.test.assertTrue

class ClientTest {

    @Test
    fun testAddresses() {
        println(InetAddress.getLocalHost())
        println(InetAddress.getByName("localhost"))
        println(InetAddress.getByName("127.0.0.1"))
        println(InetAddress.getByName("mail.ru"))
    }
    @Test
    fun testClient() = runTest {
        initCrypto()
        Log.connectConsole(Log.Level.DEBUG)
        data class Session(
            var data: String
        )

        val cmdSave by command<String,Unit>()
        val cmdLoad by command<Unit,String>()

        val cli = KiloInterface<Session>().apply {
            onConnected { session.data = "start" }
            on(cmdSave) { session.data = it }
            on(cmdLoad) {
                println("load!")
                session.data }
        }
        val server = KiloServer(cli, acceptTcpDevice(17101)) { Session("unknown")}
        val client = KiloClient<Unit> {
            connect { connectTcpDevice("localhost:17101") }
        }
        println(client.call(cmdLoad))

        assertEquals("start", client.call(cmdLoad))
        client.call(cmdSave, "foobar")
        assertEquals("foobar", client.call(cmdLoad))
        server.close()
    }

    @Test
    fun webSocketTest() = runTest {
        initCrypto()
//        fun Application.
        val cmdGetFoo by command<Unit,String>()
        val cmdSetFoo by command<String,Unit>()
        val cmdCheckConnected by command<Unit,Boolean>()

        Log.connectConsole(Log.Level.DEBUG)

        data class Session(var foo: String="not set")
        val serverInterface = KiloInterface<Session>().apply {
            var connectedCalled = false
            onConnected { connectedCalled = true }
            on(cmdGetFoo) { session.foo }
            on(cmdSetFoo) { session.foo = it }
            on(cmdCheckConnected) { connectedCalled }
        }
//       val server = setupWebsoketServer()
        val ns: NettyApplicationEngine = embeddedServer(Netty, port = 8080, host = "0.0.0.0", module = {
            setupWebsocketServer(serverInterface) { Session() }
        }).start(wait = false)

        val client = websocketClient<Unit>("ws://localhost:8080/kp")
        println(1)
        assertEquals(true, client.call(cmdCheckConnected))
        assertTrue { client.state.value }
        println(2)
        assertEquals("not set", client.call(cmdGetFoo))
        println(3)
        client.call(cmdSetFoo, "foo")
        println(4)
        assertEquals("foo", client.call(cmdGetFoo))
        println(5)

        client.close()
        ns.stop()
        println("stopped server")
        println("closed client")
    }
}